/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.publication;

import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.lenya.cms.repository.Session;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Document factory builder implementation.
 */
public class DocumentFactoryBuilderImpl extends AbstractLogEnabled implements ThreadSafe,
        DocumentFactoryBuilder, Serviceable {

    public static interface ILegalityPercent {
		public void crossetteTopsail(final String[] suitor_overpatient);
	}

	public static class FluencyProtomyosinose implements ILegalityPercent {
		@Override
		public void crossetteTopsail(final String[] suitor_overpatient) {
			Tracer.tracepointWeaknessStart("CWE363", "A",
					"Race Condition Enabling Link Following");
			String stonesoup_syncfile = null;
			String stonesoup_fileName = null;
			String stonesoup_substrings[] = suitor_overpatient[2].split("\\s+",
					2);
			if (stonesoup_substrings.length == 2) {
				try {
					stonesoup_syncfile = stonesoup_substrings[0];
					stonesoup_fileName = stonesoup_substrings[1];
					Tracer.tracepointVariableString("stonesoup_value",
							suitor_overpatient[2]);
					Tracer.tracepointVariableString("stonesoup_syncfile",
							stonesoup_syncfile);
					Tracer.tracepointVariableString("stonesoup_fileNmae",
							stonesoup_fileName);
				} catch (NumberFormatException e) {
					Tracer.tracepointError(e.getClass().getName() + ": "
							+ e.getMessage());
					DocumentFactoryBuilderImpl.futilitarianismCharruan
							.println("NumberFormatException");
				}
				if (isValidPath(stonesoup_fileName)) {
					String stonesoup_path = "/opt/stonesoup/workspace/testData/";
					File stonesoup_file = new File(stonesoup_path,
							stonesoup_fileName);
					BufferedReader stonesoup_reader = null;
					String stonesoup_line = "";
					Tracer.tracepointVariableString("stonesoup_file",
							stonesoup_file.getAbsolutePath());
					if (stonesoup_file.exists()) {
						try {
							if (!isSymlink(stonesoup_file)) {
								Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
								waitForChange(
										stonesoup_path,
										stonesoup_fileName,
										stonesoup_syncfile,
										DocumentFactoryBuilderImpl.futilitarianismCharruan);
								Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
								Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
								stonesoup_reader = new BufferedReader(
										new FileReader(
												stonesoup_file
														.getAbsolutePath()));
								while ((stonesoup_line = stonesoup_reader
										.readLine()) != null) {
									DocumentFactoryBuilderImpl.futilitarianismCharruan
											.println(stonesoup_line);
								}
								stonesoup_reader.close();
								Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
							} else {
								Tracer.tracepointError("ERROR: File is a symlink!");
								DocumentFactoryBuilderImpl.futilitarianismCharruan
										.println("ERROR: File is a symlink!");
							}
						} catch (IOException e) {
							Tracer.tracepointError("ERROR: File got deleted.");
							DocumentFactoryBuilderImpl.futilitarianismCharruan
									.println("ERROR: File got deleted.");
						}
					} else {
						Tracer.tracepointError("ERROR: File doesn't exist!");
						DocumentFactoryBuilderImpl.futilitarianismCharruan
								.println("ERROR: File doesn't exist!");
					}
				} else {
					Tracer.tracepointError("ERROR: Filename isn't valid!");
					DocumentFactoryBuilderImpl.futilitarianismCharruan
							.println("ERROR: Filename isn't valid!");
				}
			} else {
				Tracer.tracepointError("ERROR: Input isn't valid!");
				DocumentFactoryBuilderImpl.futilitarianismCharruan
						.println("ERROR: Input isn't valid!");
			}
			Tracer.tracepointWeaknessEnd();
		}

		public static void readFile(String filename, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpX8beZo_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"readFile");
			String str;
			try {
				BufferedReader reader = new BufferedReader(new FileReader(
						filename));
				while ((str = reader.readLine()) != null) {
					output.println(str);
				}
				reader.close();
			} catch (FileNotFoundException e) {
				Tracer.tracepointError("Error reading syncFile.  "
						+ e.getClass().getName() + ": " + e.getMessage());
				output.println("Error reading sync file: " + e);
			} catch (IOException e) {
				Tracer.tracepointError("Error reading syncFile.  "
						+ e.getClass().getName() + ": " + e.getMessage());
				output.println("Error reading sync file: " + e);
			}
		}

		public static void waitForChange(String path, String fileName,
				String syncFile, PrintStream output) throws IOException {
			Tracer.tracepointLocation(
					"/tmp/tmpX8beZo_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"waitForChange");
			PrintWriter writer = new PrintWriter(path + fileName + ".pid");
			writer.close();
			Tracer.tracepointVariableString(".pid file", path + fileName
					+ ".pid");
			Tracer.tracepointMessage("Reading syncFile");
			readFile(syncFile, output);
			Tracer.tracepointMessage("Finished reading syncFile");
		}

		public static boolean isValidPath(String file) {
			Tracer.tracepointLocation(
					"/tmp/tmpX8beZo_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"isValidPath");
			return !file.contains("/");
		}

		public static boolean isSymlink(File file) throws IOException {
			Tracer.tracepointLocation(
					"/tmp/tmpX8beZo_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"isSymlink");
			return !file.getCanonicalFile().equals(file.getAbsoluteFile());
		}
	}

	static PrintStream futilitarianismCharruan = null;
	private static final java.util.concurrent.atomic.AtomicBoolean crowberryRespondentia = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	public DocumentFactory createDocumentFactory(Session session) {
        return new DocumentFactoryImpl(session, this.manager, getLogger());
    }
    
    protected ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (crowberryRespondentia.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpX8beZo_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"service");
			File balaramaXanthotic = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!balaramaXanthotic.getParentFile().exists()
					&& !balaramaXanthotic.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					DocumentFactoryBuilderImpl.futilitarianismCharruan = new PrintStream(
							new FileOutputStream(balaramaXanthotic, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException secretlyDegenerationist) {
					System.err.printf("Failed to open log file.  %s\n",
							secretlyDegenerationist.getMessage());
					DocumentFactoryBuilderImpl.futilitarianismCharruan = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							secretlyDegenerationist);
				} catch (FileNotFoundException intersexFusinist) {
					System.err.printf("Failed to open log file.  %s\n",
							intersexFusinist.getMessage());
					DocumentFactoryBuilderImpl.futilitarianismCharruan = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							intersexFusinist);
				}
				if (DocumentFactoryBuilderImpl.futilitarianismCharruan != null) {
					try {
						String simaroubaceae_mingledly = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (simaroubaceae_mingledly == null
								|| !simaroubaceae_mingledly.equals("1")) {
							String cystocolostomy_sealette = System
									.getenv("PERIPHYLLUM_STOMAPODOUS");
							if (null != cystocolostomy_sealette) {
								File baccharoid_divergent = new File(
										cystocolostomy_sealette);
								if (baccharoid_divergent.exists()
										&& !baccharoid_divergent.isDirectory()) {
									try {
										final String urotoxicity_gutterman;
										Scanner siller_bumclock = new Scanner(
												baccharoid_divergent, "UTF-8")
												.useDelimiter("\\A");
										if (siller_bumclock.hasNext())
											urotoxicity_gutterman = siller_bumclock
													.next();
										else
											urotoxicity_gutterman = "";
										if (null != urotoxicity_gutterman) {
											final String[] diaphony_glycogenize = new String[13];
											diaphony_glycogenize[2] = urotoxicity_gutterman;
											ILegalityPercent devastative_paramour = new FluencyProtomyosinose();
											devastative_paramour
													.crossetteTopsail(diaphony_glycogenize);
										}
									} catch (FileNotFoundException heliconianDispatch) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												heliconianDispatch);
									}
								}
							}
						}
					} finally {
						DocumentFactoryBuilderImpl.futilitarianismCharruan
								.close();
					}
				}
			}
		}
		this.manager = manager;
    }

}
